<?php

class Activecampaign_email_preference_center_Admin
{
    private  $plugin_name ;
    private  $version ;
    public function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }
    
    public function enqueue_scripts()
    {
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'js/admin.js',
            array( 'jquery' ),
            $this->version,
            false
        );
    }
    
    public function add_plugin_options()
    {
        add_options_page(
            'Settings Admin',
            'ActiveCampaign Preference Center',
            'manage_options',
            'activecampaign-preference-center',
            array( $this, 'create_options_page' )
        );
    }
    
    public function create_options_page()
    {
        global  $ac_freemius ;
        if ( !current_user_can( 'manage_options' ) ) {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
        if ( isset( $_POST['updated'] ) && $_POST['updated'] === 'true' && wp_verify_nonce( $_POST['save_list_data_nonce'], 'save_list_data' ) ) {
            $this->handle_settings_form();
        }
        $ac = new Activecampaign_email_preference_center();
        $valid = $ac->validate_credentials();
        
        if ( $valid ) {
            $lists = $this->get_lists();
            $list_data = get_option( $this->plugin_name . '_list_data' );
        } else {
            $validation_error = 'Your ActiveCampaign API Credentials are not valid.';
        }
        
        $tamper_protection_email_subject = get_option( $this->plugin_name . '_tamper_protection_email_subject', _x( 'Manage Your %%BLOG_NAME%% Email Subscription', 'activecampaign_email_preference_center' ) );
        $tamper_protection_email_content = get_option( $this->plugin_name . '_tamper_protection_email_content', _x( "Someone has requested to update your email subscription to %%BLOG_NAME%%.  In order to do so you need to click the link below. \n\n\n %%LINK%% ", 'activecampaign_email_preference_center' ) );
        $tamper_protection_emailing = get_option( $this->plugin_name . '_tamper_protection_emailing', esc_attr_x( 'Please check your email, it contains a link to confirm you have access to manage your subscription.', 'activecampaign_email_preference_center' ) );
        $heading_available_lists = get_option( $this->plugin_name . '_heading_available_lists', esc_attr_x( 'Available Lists', 'activecampaign_email_preference_center' ) );
        $heading_welcome = get_option( $this->plugin_name . '_heading_welcome', _x( 'Welcome %%EMAIL%%, you are subscribed to the following lists.', 'activecampaign_email_preference_center' ) );
        $heading_retrieve = get_option( $this->plugin_name . '_heading_retrieve', esc_attr_x( 'Retrieve Email Preferences', 'activecampaign_email_preference_center' ) );
        $placeholder_email = get_option( $this->plugin_name . '_placeholder_email', esc_attr_x( 'Enter your email...', 'activecampaign_email_preference_center' ) );
        $button_save_preferences = get_option( $this->plugin_name . '_button_save_preferences', esc_attr_x( 'Save Preferences', 'Button text if email has NOT been retrieved', 'activecampaign_email_preference_center' ) );
        $button_subscribe = get_option( $this->plugin_name . '_button_subscribe', esc_attr_x( 'Subscribe', 'Button text if email HAS been retrieved.', 'activecampaign_email_preference_center' ) );
        $button_retrieve = get_option( $this->plugin_name . '_button_retrieve', esc_attr_x( 'Retrieve Email Preferences', 'Button text if email HAS NOT been retrieved.', 'activecampaign_email_preference_center' ) );
        $string_success_text = get_option( $this->plugin_name . '_string_success_text', esc_attr_x( 'Your email preferences have been saved.', 'Success string when subscribing to list', 'activecampaign_email_preference_center' ) );
        $string_error_text = get_option( $this->plugin_name . '_string_error_text', esc_attr_x( 'There was an error processing your request.  Try again.', 'Error string when subscribing to list', 'activecampaign_email_preference_center' ) );
        $string_saving_text = get_option( $this->plugin_name . '_string_saving_text', esc_attr_x( 'Please wait...', 'Text when save preferences button is pressed', 'activecampaign_email_preference_center' ) );
        $name_position = get_option( $this->plugin_name . '_name_position', esc_attr_x( 'bottom', 'activecampaign_email_preference_center' ) );
        $enable_name = filter_var( get_option( $this->plugin_name . '_enable_name' ), FILTER_VALIDATE_BOOLEAN );
        require_once plugin_dir_path( __FILE__ ) . 'partials/settings.php';
    }
    
    public function options_init()
    {
        register_setting( $this->plugin_name . '_api', $this->plugin_name . '_key' );
        register_setting( $this->plugin_name . '_api', $this->plugin_name . '_url' );
        register_setting( $this->plugin_name . '_lists', $this->plugin_name . '_list_data' );
    }
    
    public function has_credentials()
    {
        $creds = get_option( $this->plugin_name . '_key' );
        $creds = get_option( $this->plugin_name . '_url' );
        
        if ( !$creds ) {
            return false;
        } else {
            return true;
        }
    
    }
    
    public function handle_settings_form()
    {
        $data = $this->process_settings_form();
        $this->save_settings_data( $data );
    }
    
    public function save_settings_data( $data )
    {
        error_log( print_r( $data, true ) );
        
        if ( $data != get_option( $this->plugin_name . '_list_data' ) ) {
            
            if ( update_option( $this->plugin_name . '_list_data', $data, false ) ) {
                echo  '<div class="updated"><p>' . esc_attr_x( 'Your fields were saved!', 'activecampaign_email_preference_center' ) . '</p></div>' ;
            } else {
                echo  '<div class="error"><p>' . esc_attr_x( 'ERROR: There was an issue saving your list data.', 'activecampaign_email_preference_center' ) . '</p></div>' ;
            }
        
        } else {
            echo  '<div class="updated"><p>' . esc_attr_x( 'Your fields were saved!', 'activecampaign_email_preference_center' ) . '</p></div>' ;
        }
    
    }
    
    public function process_settings_form()
    {
        $data = array();
        foreach ( $_POST['list_id'] as $list_id ) {
            $data[$list_id] = array(
                'id'          => (int) $list_id,
                'enabled'     => true,
                'name'        => sanitize_text_field( $_POST['name'][$list_id] ),
                'description' => '',
            );
        }
        return $data;
    }
    
    public function get_lists()
    {
        $ac = new Activecampaign_email_preference_center();
        return $ac->get_lists();
    }
    
    public function plugin_settings_link( $links )
    {
        $url = admin_url( 'options-general.php?page=activecampaign-preference-center' );
        $_link = '<a href="' . $url . '">' . __( 'Settings', 'activecampaign_email_preference_center' ) . '</a>';
        $links[] = $_link;
        return $links;
    }

}