<?php

class Activecampaign_email_preference_center {

	protected $loader;
	protected $plugin_name;
	protected $version;

	public function __construct() {

		if ( defined( 'ACEPC_PLUGIN_VERSION' ) ) {
			$this->version = ACEPC_PLUGIN_VERSION;
		} else {
			$this->version = '2.0.2';
		}
		$this->plugin_name = 'activecampaign_email_preference_center';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	public function get_plugin_name() {
		return $this->plugin_name;
	}

	public function get_version() {
		return $this->version;
	}

	public function run() {
		$this->loader->run();
	}

	public function get_loader() {
		return $this->loader;
	}

	public function get_lists() {
		$return = array();
		$lists  = $this->get( 'lists' );

		foreach ( $lists->lists as $list ) {
			if ( is_object( $list ) ) {
				$return[ $list->id ] = array(
					'id'   => $list->id,
					'name' => $list->name
				);
			}
		}

		return $return;
	}

	public function get_contact( $email ) {

		$contact = $this->get( 'contacts', array( 'email' => $email ) );

		if ( empty( $contact->contacts ) ) {
			$contact = $this->create_contact( sanitize_email( $email ) );

			$return = [
				'id'         => $contact->id,
				'first_name' => $contact->firstName,
				'last_name'  => $contact->lastName,
				'email'      => $contact->email
			];

			$list_url = $contact->links->contactLists;
		} else {
			$return = [
				'id'         => $contact->contacts[0]->id,
				'first_name' => $contact->contacts[0]->firstName,
				'last_name'  => $contact->contacts[0]->lastName,
				'email'      => $contact->contacts[0]->email
			];

			$list_url = $contact->contacts[0]->links->contactLists;
		}

		$get_lists = $this->get( $list_url );

		$lists = [];

		if ( ! is_null( $get_lists ) ) {
			foreach ( $get_lists->contactLists as $list ) {
				if ( $list->status === '1' ) {
					$lists[] = $list->list;
				}
			}
		}


		$return['lists'] = $lists;

		return $return;

	}

	public function create_contact( $email ) {
		$contact = $this->post( 'contacts', [ 'contact' => [ 'email' => $email ] ] );

		return $contact;
	}

	public function update_contact( $data ) {
		$contact = $this->post( 'contact/sync', [
			'contact' => [
				'email'     => $data['email'],
				'firstName' => $data['first_name'],
				'lastName'  => $data['last_name']
			]
		] );
		return $contact;
	}


	public function update_contact_lists( $data ) {
		return $this->post( 'contactLists', [ 'contactList' => $data ] );
	}

	public function validate_credentials(){
		$api = $this->api( '' );
		$url = $api['url'];
		$response = wp_remote_get( $url, $api['args'] );

		if($response['response']['code'] === 200){
			return true;
		}

		return false;

	}

	protected function get( $endpoint, $params = array() ) {

		$api = $this->api( $endpoint );
		$url = $api['url'];

		if ( count( $params ) > 0 ) {
			$url = $api['url'] . '?' . http_build_query( $params );
		}

		$response = wp_remote_get( $url, $api['args'] );
		$body     = wp_remote_retrieve_body( $response );
		return json_decode( $body );

	}

	protected function post( $endpoint, $params = array() ) {

		$api = $this->api( $endpoint );
		$url = $api['url'];

		$args = array_merge( $api['args'], [ 'body' => json_encode( $params ) ] );
		$response = wp_remote_post( $url, $args );
		$body     = wp_remote_retrieve_body( $response );

		return json_decode( $body );

	}


	protected function api( $endpoint ) {
		$ac_url     = esc_attr( get_option( $this->plugin_name . '_url' ) );
		$ac_key     = esc_attr( get_option( $this->plugin_name . '_key' ) );
		$ac_version = '/api/3/';
		$url        = $ac_url . $ac_version . $endpoint;

		if ( strpos( $endpoint, $ac_url ) !== false ) {
			$url = $endpoint;
		}

		$args = [ 'headers' => [ 'Api-Token' => $ac_key ] ];

		return array( 'args' => $args, 'url' => $url );


	}

	private function load_dependencies() {

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/loader.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/i18n.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/public.php';

		$this->loader = new Activecampaign_email_preference_center_Loader();

	}

	private function set_locale() {

		$plugin_i18n = new Activecampaign_email_preference_center_i18n();
		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	private function define_admin_hooks() {

		$plugin_admin = new Activecampaign_email_preference_center_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_plugin_options' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'options_init' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_filter( 'plugin_action_links_activecampaign-email-preference-center/activecampaign-email-preference-center.php', $plugin_admin, 'plugin_settings_link', 10, 2 );
	}

	private function define_public_hooks() {

		$plugin_public = new Activecampaign_email_preference_center_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_public, 'init' );
		$this->loader->add_action( 'wp_ajax_update_preferences', $plugin_public, 'ajax_update_preferences' );
		$this->loader->add_action( 'wp_ajax_nopriv_update_preferences', $plugin_public, 'ajax_update_preferences' );

	}

}
