<?php

class Activecampaign_email_preference_center_Public
{
    private  $plugin_name ;
    private  $version ;
    public function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }
    
    public function init()
    {
        $this->register_shortcodes();
    }
    
    public function register_shortcodes()
    {
        add_shortcode( 'ac-preference-center', array( $this, 'preference_center_shortcode' ) );
    }
    
    public function enqueue_scripts()
    {
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'js/public.js',
            array( 'jquery' ),
            $this->version,
            false
        );
        $string_success_text = get_option( $this->plugin_name . '_string_success_text', esc_attr_x( 'Your email preferences have been saved.', 'Success string when subscribing to list', 'activecampaign_email_preference_center' ) );
        $string_error_text = get_option( $this->plugin_name . '_string_error_text', esc_attr_x( 'There was an error processing your request.  Try again.', 'Error string when subscribing to list', 'activecampaign_email_preference_center' ) );
        $string_saving_text = get_option( $this->plugin_name . '_string_saving_text', esc_attr_x( 'Please wait...', 'Text when save preferences button is pressed', 'activecampaign_email_preference_center' ) );
        $button_save_preferences = get_option( $this->plugin_name . '_button_save_preferences', esc_attr_x( 'Save Preferences', 'Button text if email has NOT been retrieved', 'activecampaign_email_preference_center' ) );
        $tamper_protection = filter_var( get_option( $this->plugin_name . '_tamper_protection' ), FILTER_VALIDATE_BOOLEAN );
        wp_localize_script( $this->plugin_name, 'ac_ajax', [
            'ajax_url'                 => admin_url( 'admin-ajax.php' ),
            'update_nonce'             => wp_create_nonce( 'update_nonce' ),
            'success_text'             => $string_success_text,
            'error_text'               => $string_error_text,
            'saving_text'              => $string_saving_text,
            'save_text'                => $button_save_preferences,
            'enable_tamper_protection' => $tamper_protection,
        ] );
    }
    
    public function preference_center_shortcode( $atts )
    {
        ob_start();
        $args = shortcode_atts( array(
            'show_action_text'       => false,
            'show_list_descriptions' => true,
            'show_welcome_text'      => true,
            'list_ids'               => null,
        ), $atts, 'ac-preference-center' );
        $show_action_text = filter_var( $args['show_action_text'], FILTER_VALIDATE_BOOLEAN );
        $show_list_descriptions = filter_var( $args['show_list_descriptions'], FILTER_VALIDATE_BOOLEAN );
        $show_welcome_text = filter_var( $args['show_welcome_text'], FILTER_VALIDATE_BOOLEAN );
        $sending_email = false;
        $email = $this->getEmailFromGet();
        $ac = new Activecampaign_email_preference_center();
        $contact = $ac->get_contact( $email );
        $subscriber_id = ( !empty($contact['id']) ?: $contact['id'] );
        $lists = $this->get_lists();
        $has_credentials = $this->has_credentials();
        $subscribed_lists = [];
        if ( !is_null( $email ) && isset( $contact['lists'] ) ) {
            $subscribed_lists = $contact['lists'];
        }
        require plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/preference-center.php';
        return ob_get_clean();
    }
    
    public function get_lists()
    {
        $lists = get_option( $this->plugin_name . '_list_data' );
        $lists = ( !$lists ? array() : $lists );
        return $lists;
    }
    
    public function has_credentials()
    {
        $creds = get_option( $this->plugin_name . '_key' );
        $creds = get_option( $this->plugin_name . '_url' );
        
        if ( !$creds ) {
            return false;
        } else {
            return true;
        }
    
    }
    
    public function get_subscribed_lists( $contact )
    {
        $subscribed_lists = array();
        if ( is_object( $contact ) && $contact->result_code != 0 ) {
            foreach ( $contact->lists as $list ) {
                if ( isset( $list->status ) && $list->status == 1 ) {
                    $subscribed_lists[] = $list->listid;
                }
            }
        }
        return $subscribed_lists;
    }
    
    public function ajax_update_preferences()
    {
        
        if ( check_ajax_referer( 'update_nonce', 'security' ) ) {
            $tamper_protection = (bool) get_option( $this->plugin_name . '_tamper_protection' );
            $enable_name = (bool) get_option( $this->plugin_name . '_enable_name' );
            parse_str( $_POST['data'], $data );
            $current_subscribed_list_ids = unserialize( urldecode( $data['current_subscribed_list_ids'] ) );
            if ( !isset( $data['subscribed_list_ids'] ) ) {
                $data['subscribed_list_ids'] = array();
            }
            $submitted_list_ids = $data['subscribed_list_ids'];
            $ac = new Activecampaign_email_preference_center();
            $contact = $ac->get_contact( sanitize_email( $data['email'] ) );
            if ( ac_freemius()->is__premium_only() && ac_freemius()->can_use_premium_code() && $enable_name ) {
                if ( ac_freemius()->can_use_premium_code() ) {
                    if ( $enable_name ) {
                        $ac->update_contact( [
                            'first_name' => sanitize_text_field( $data['first_name'] ),
                            'last_name'  => sanitize_text_field( $data['last_name'] ),
                            'email'      => sanitize_email( $data['email'] ),
                        ] );
                    }
                }
            }
            $lists = [];
            /**
             * Create array of our current subscribed lists and statuses
             */
            foreach ( $current_subscribed_list_ids as $current_subscribed_list_id ) {
                $status = ( in_array( $current_subscribed_list_id, $submitted_list_ids ) ? 1 : 2 );
                //1 = subscribed 2 = un-subscribed
                $lists[$current_subscribed_list_id] = $status;
            }
            /**
             * Loop through our submitted list IDs and mark them down as subscribed.
             */
            foreach ( $submitted_list_ids as $submitted_list_id ) {
                $lists[$submitted_list_id] = 1;
            }
            foreach ( $lists as $id => $status ) {
                $sync = $ac->update_contact_lists( [
                    'list'    => $id,
                    'contact' => $contact['id'],
                    'status'  => $status,
                ] );
            }
            if ( empty($sync->contacts) ) {
                wp_send_json( 'Error updating list status.' );
            }
            //We re grab the contact info from AC to include newly subscribed lists.
            $contact = $ac->get_contact( sanitize_email( $data['email'] ) );
            wp_send_json( urlencode( serialize( $contact['lists'] ) ), 200 );
        }
        
        die;
    }
    
    protected function getEmailFromGet()
    {
        
        if ( isset( $_GET['email'] ) && !empty($_GET['email']) && sanitize_email( $_GET['email'] ) ) {
            /** Since we are using $_GET the PHP will convert + automatically to a blank space when coming from $_GET
             * In order to handle GMAIL alias addresses we need to convert those spaces back to + */
            $handle_plus = str_replace( ' ', '+', $_GET['email'] );
            return sanitize_email( $handle_plus );
        }
        
        return null;
    }
    
    protected function get_subscriber_id_from_email( $email )
    {
        $ac = new Activecampaign_email_preference_center();
        $subscriber_id = null;
        $contact = $ac->get_contact( $email );
        if ( isset( $contact->subscriberid ) ) {
            return $contact->subscriberid;
        }
        return false;
    }
    
    protected function is_valid_subscriber_id( $subscriber_id )
    {
        if ( isset( $_GET['id'] ) && !empty($_GET['id']) && is_numeric( $_GET['id'] ) ) {
            if ( $_GET['id'] == $subscriber_id ) {
                return true;
            }
        }
        return false;
    }
    
    protected function get_contact_id( $email )
    {
        $ac = new Activecampaign_email_preference_center();
        $contact = $ac->get_contact( sanitize_email( $email ) );
        return $contact['id'];
    }

}